<?php
/**
 * @file
 * Contains \Drupal\GeneratorUI\Controller\GeneratorController.
 *
 */

namespace Drupal\generator_ui\Controller;

//Use the necessary classes
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenDialogCommand;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormState;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\Core\Url;
use Drupal\generator_ui\Form\Generator;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\JsonResponse;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;

// GeneratorController class manages all the common features between  generators
class GeneratorController extends ControllerBase {

  protected $twig;
  protected $id;

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Template\TwigEnvironment $twig
   *   used to load templates from the file system or other locations.
   */
  public function __construct(TwigEnvironment $twig) {
    $this->twig = $twig;
  }

  /**
   * Instantiation the twig service which we can use in this class.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container .
   *   container dependency injection
   * @return
   *   The twig service.
   */
  public static function create(ContainerInterface $container) {

    return new static(
      $container->get('twig')

    );
  }

  /**
   * List of all modules.
   * @param  $path
   *   path of the file
   * @return array()
   *   Recuperate all files in the directory.
   */
  public static function listInfoFiles() {
    //path of templates files
    //find directories and files in template directory
    $dir_modules = DRUPAL_ROOT . '/modules';

    $iter = new \RecursiveDirectoryIterator($dir_modules);
    foreach (new \RecursiveIteratorIterator($iter) as $file) {
      //if the file has a twig extension
      if (strpos($file->getFilename(), '.info.yml')) {
        // declare an array contains the list of modules
        $file_name = substr($file->getFilename(), 0, strpos($file->getFilename(), '.'));
        $list_generators[] = $file_name;
      }
    }
    return $list_generators;
  }


  /**
   * List of files in directory.
   *
   * @return array()
   *   Recuperate all modules in the project.
   */
  public static function listModules() {
    if ($dir = opendir(DRUPAL_ROOT . '/modules')) {
      while ($file = readdir($dir)) {
        $string = $file;
        $names = explode(",", $string);
        foreach ($names as $name) {
          if (strpos($name, '.') == FALSE && strlen($name) > 2) {
            $listModule[] = $name;
          }
        }
      }
      closedir($dir);
      return $listModule;
    }
  }


  /**
   * Verify the file exstance in a directory.
   *
   * @param string $pathModule
   *   Path of modules.
   * @param string $nameModule
   *  Module name.
   * @param string $type
   *   The file type.Ex.info,routing,controller...
   * @return boolean
   *   return 1 if the modules there,else return 0.
   */
  public static function exist($module_name, $file_name) {
    $name_module = DRUPAL_ROOT . '/' . drupal_get_path('modules', $module_name);
    $dir_modules = $name_module . '/modules/';
    $lists_modules = scandir($dir_modules);
    $exist = 0;
    foreach ($lists_modules as $list) {
      if ($list == $file_name) {
        $exist = 1;
      }
    }
    return $exist;
  }

// return true or false , if the file exist then true else false
  public function exist_file($file, $module_name) {
    $list_files = self::getListFiles($module_name);
    $exist = FALSE;
    if ($list_files != NULL) {
      foreach ($list_files as $list_file) {
        if ($list_file == $file) {
          $exist = TRUE;
        }
      }
    }
    return $exist;
  }

  //get All files in the new modules
  public static function getListFiles($module_name) {
    $new_module = drupal_get_path('module', $module_name);
    if ($new_module != NULL) {
      $iter = new \RecursiveDirectoryIterator($new_module);
      foreach (new \RecursiveIteratorIterator($iter) as $file) {
        // declare an array contains the list of all files
        $list[] = $file->getPath() . '/' . $file->getFilename();
      }
    }
    return $list;
  }

  /**
   * it used to do the autocomplete path for all modules.
   *
   * @return \Symfony\Component\HttpFoundation\JsonResponse
   *   return list of modules in the project with json format.
   */
  public function autocomplete() {
    $listModules = self::listModules();
    return new JsonResponse($listModules);
  }


  /**
   * it used for the page presentation of Generator UI.
   *
   * @return markup
   *   return markup which contains the efficiency of Generator Ui and available code generators.
   */
  public function helpGenerator() {
    $output = '<h2>' . t('What\'s this ?') . '</h2>';
    $output .= '<li>' . '<p>' . t('This modules is for Drupal 8 developpers (from beginners to advanced ones) only. There will never be any "stable" release as it is intended to be a dev tool under constant development.') . '</li>' . '</p>';
    $output .= '<li>' . '<p>' . t('Generator UI allows you to develop faster by allowing you to generate code from template files packaged into the modules within your Drupal installation. As an example you can therefore create a modules with a path and its permission directly into your drupal installation. Up to you to change the default "hello world" into whatever you want.') . '</li>' . '</p>';
    $output .= '<li>' . '<p>' . t('The modules has also its help mode that brings a lot of comments on the code to help you understand how to develop with Drupal 8.') . '</li>' . '</p>';
    $output .= '<h2>' . t('Module Efficiency') . '</h2>';
    $output .= '<li>' . '<p>' . t('The code generators facilitates the generation of multiple codes and new concepts in Drupal 8, as the forms, entities, controllers and others ...') . '</li>' . '</p>';
    $output .= '<li>' . '<p>' . t('This generator also offers the download of code you have already generated..') . '</li>' . '</p>';
    $output .= '<li>' . '<p>' . t('Finally, during the installation of this modules, you can integrate the code you generated in your modules.') . '</li>' . '</p>';
    $output .= '<h2>' . t('Available code generators') . '</h2>';
    $output .= '<p>';
    $output .= '<li>' . t('<strong> <a href="!info">Generate .info.yml</a></strong>', array('!info' => \Drupal::url('generator.info'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!permissions">Generate .permissions.yml</a></strong>', array('!permissions' => \Drupal::url('generator.permissions'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!routing">Generate routing with Controller/Form</a></strong>', array('!routing' => \Drupal::url('generator.routing'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!install">Generate .install</a></strong>', array('!install' => \Drupal::url('generator.install'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!links_task">Generate module.links.task.yml</a></strong>', array('!links_task' => \Drupal::url('generator.links_task'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!form">Generate Form</a></strong>', array('!form' => \Drupal::url('generator.form'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!config_form">Generate Config Form</a></strong>', array('!config_form' => \Drupal::url('generator.configform'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!ajaxform">Generate Form with Ajax</a></strong>', array('!ajaxform' => \Drupal::url('generator.ajaxform'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!block">Generate Block</a></strong>', array('!block' => \Drupal::url('generator.block'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!formatter">Generate Plugin Formatter</a></strong>', array('!formatter' => \Drupal::url('generator.formatter'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!field_widget">Generate Plugin FieldWidget</a></strong>', array('!field_widget' => \Drupal::url('generator.field_widget'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!field_type">Generate Plugin FieldType</a></strong>', array('!field_type' => \Drupal::url('generator.twig'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!formatter">Generate Plugin Formatter</a></strong>', array('!formatter' => \Drupal::url('generator.formatter'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!generator">Generate Generator</a></strong>', array('!generator' => \Drupal::url('generator.generator'))) . '</li>';
    $output .= '<li>' . t('<strong> <a href="!mail">Generate example of sending mail</a></strong>', array('!mail' => \Drupal::url('generator.mail'))) . '</li>';
    $output .= '<br/>' . '<p>' . t('Link to the modules on Drupal.org :') . t('<strong> <a href="https://www.drupal.org/project/generator_ui">https://www.drupal.org/project/generator_ui</a></strong>') . '</p>';
    $output .= '</p>';
    return array(
      '#markup' => $output,
    );
  }
  protected function loadFromTwig($parameters, $pathTemplate) {
    $template = $this->twig->loadTemplate($pathTemplate);
    $output = array(
      $template->render($parameters)
    );
    $outputFinal = implode("\n", $output);

    return $outputFinal;
  }

 // Clear all caches.
  public function clearCache() {
    drupal_flush_all_caches();
    drupal_set_message(t('Caches cleared successfully.'));
    return new RedirectResponse('/');
  }


}

