<?php
/**
 * @file
 * Contains \Drupal\generator_ui\Form\GeneratorForm.
 *
 */

namespace Drupal\generator_ui\Form;

//Use the necessary classes
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Template\TwigEnvironment;
use Drupal\generator_ui\Controller\GeneratorController;

/**
 * Class GeneratorForm
 * @package Drupal\generator_ui\Form
 */
class Generator extends FormBase {

  protected $twig;

  /**
   * The constructor.
   *
   * @param \Drupal\Core\Template\TwigEnvironment $twig
   *   used to load templates from the file system or other locations.
   */
  public function __construct(TwigEnvironment $twig) {
    $this->twig = $twig;
  }

  /**
   * Instantiation the twig service which we can use in this class.
   *
   * @param \Symfony\Component\DependencyInjection\ContainerInterface $container .
   *   container dependency injection
   * @return
   *   The twig service.
   */
  public static function create(ContainerInterface $container) {

    return new static(
      $container->get('twig')

    );
  }

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['module_name'] = array(
      '#type' => 'textfield',
      '#default_value' => 'module',
      '#autocomplete_route_name' => 'generator.autocomplete',
      '#title' => t('Module machine name'),
      '#required' => TRUE,
      '#element_validate' => array(array(get_class($this), 'validate_module')),
      "#weight" => -1
    );
    $form['checkbox_name'] = array(
      '#type' => 'checkbox',
      '#title' => t('On/Off'),
      '#description' => t('You can add comments when generating code !'),
      "#weight" => -2
    );
    $form['btn_generate'] = array(
      '#type' => 'button',
      '#executes_submit_callback' => TRUE,
      '#button_type' => 'submit',
      '#value' => t('Generate code(s)'),
      '#ajax' => array(
        'callback' => '::generateCodes',
        'wrapper' => 'ajax_div',
      ),
    );
    $form['btn_create'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Create files'),
      '#submit' => array('::createFiles')
    );
    $form['btn_download'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Download files'),
      '#submit' => array('::downloadFiles')
    );
    $form['ajx'] = array(
      '#type' => 'textarea',
      '#resizable' => TRUE,
      '#rows' => 15,
      '#attributes' => array('onclick' => 'this.select()'),
      '#cols' => 10,
      '#prefix' => '<div id="ajax_div">',
    );
    return $form;
  }



  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    $module_name = $form_state->getValue('module_name');
    $all_files_to_create = $this->pathFiles($form_state);

    // Check existence of the files.
    foreach ($all_files_to_create AS $file) {
      //delete DRUPAL_ROOT from paths
      $file = str_replace(DRUPAL_ROOT . "/", "", $file);
      // If file already exists, we cannot create (do not erase previous file for now).
      if (GeneratorController::exist_file($file, $module_name)) {
        $form_state->setErrorByName('module_name', $this->t('The file ' . $file . ' already exists.'));
      }
    }
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $form_state->setRebuild();
  }

  // Validation of existence of module in modules folder
  public  function validate_module($element, FormStateInterface $form_state) {
    $module_name = $element['#value'];
    $form_state->setValueForElement($element, $module_name);
    // existence of module in list of modules
    $exist_file = GeneratorController::exist_file('modules/' . $module_name . '/' . $module_name . '.info.yml', $module_name);
    //if($this->getFormId() != 'info')
    if (!$exist_file) {
      $form_state->setError($element, t('The module " %module_name " doesn\'t exist.', array('%module_name' => $module_name)));
    }
  }

  // return all the names of textarea
  public function getTexareaNames(array &$form) {
    foreach ($form as $form_) {
      // if the type is a textarea
      if ($form_['#type'] == 'textarea') {
        //return the name of the textarea
        $list_prefix[] = $form_['#name'];
      }
    }
    return $list_prefix;
  }

  // Fill all textareas with corresponding codes.
  public function generateCodes(array &$form, FormStateInterface $form_state) {
    $module_name = $form_state->getValue('module_name');
    $params = $this->getAllValues($form_state);
    $paths_all_twig = $this->getFullPathTwig($form_state);
    $text_area = $this->is_not_null($this->getTexareaNames($form));
    for ($i = 0; $i < sizeof($paths_all_twig); $i++) {
      $input = $this->loadFromTwig($params, $paths_all_twig[$i]);
      $form[$text_area[$i]]['#value'] = $input;
      $text_area_values[$i] = $form[$text_area[$i]];
    }
    return $text_area_values;

  }

  // get right paths of all twig files
  public function path(FormStateInterface $form_state) {
    //name of all twig files
    $twig_filess = $form_state->getValue('twig_file');
    foreach ($twig_filess as $twig_file => $value) {
      $twigg_files[] = $value;
    }
    $all_twig_files = $this->getRealPathNamesTwigFiles();
    foreach ($twigg_files as $twigg_file) {
      $list = $all_twig_files[$twigg_file];
      $list_all_twig_files[] = $list;
    }
    return $list_all_twig_files;
  }

  /**
   * create the package which contain all files generated.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @param $twig_file
   *   Name of twig file declarated in templates.
   */
  public function createFiles(array &$form,FormStateInterface $form_state) {
    $params = $this->getAllValues($form_state);
    // twig files
    $twigFiles = $this->path($form_state);
    //the right path
    $files_to_create = $this->pathFiles($form_state);
    $new_files_to_create = $this->is_not_null($files_to_create);
    for ($i = 0; $i < count($new_files_to_create); $i++) {
     drupal_set_message('The File '. $new_files_to_create[$i]. ' was created successfully !');
      $this->createFile(
        $twigFiles[$i],
        $files_to_create[$i],
        $params
      );
    }
  }

  // This method extract the name of PHP File classes
  public function extractNamePhpFiles(array &$form, FormStateInterface $form_state) {
    $twig_files = $form_state->getValue('twig_file');
    // look over all twig names
    foreach ($twig_files as $twig_file => $value) {
      // if the twig names is php twig file
      if (!is_numeric($twig_file)) {
        // if the class name is not null
        if ($form_state->getValue($twig_file) != NULL) {
          // Save all class php names
          $php_file[] = $form_state->getValue($twig_file);
        }
        else {
          $php_file[] = NULL;
        }
      }
    }
    return $php_file;
  }

  /**
   * Creating files.
   *
   * @param  $pathTemplate
   *  Path of template file.
   * @param $target
   *   Target to create file.
   * @param $parameters
   *   values passed to twig files.
   */
  protected function createFile($pathTemplate, $target, $parameters, $flag = NULL) {
    if (!is_dir(dirname($target))) {
      mkdir(dirname($target), 0770, TRUE);
      chmod(dirname($target), 0770);
    }
    return file_put_contents($target, $this->loadFromTwig($parameters, $pathTemplate), $flag);
  }
  /**
   * download  the package which contain all files generated.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function downloadFiles(array &$form,FormStateInterface $form_state){
    unset($form['module_name']['#element_validate']);
    $string_to_eleminate = DRUPAL_ROOT . "/modules/";
    $files_to_create = $this->pathFiles($form_state);
    $new_files_to_create = $this->is_not_null($files_to_create);
    foreach($new_files_to_create as $new_file_to_create ){
      // eliminate the Root Directory from the file
      $new_files[] = str_replace($string_to_eleminate,"",$new_file_to_create);
    }
    // Create new zip object
    $zip = new \ZipArchive();
    // Create a temp file & open it
    $tmp_file = tempnam('.','');
    $zip->open($tmp_file, \ZipArchive::CREATE);
    $params = $this->getAllValues($form_state);
    $paths_all_twig = $this->getFullPathTwig($form_state);

    for ($i = 0; $i < count($paths_all_twig); $i++) {
      $input[] = $this->loadFromTwig($params, $paths_all_twig[$i]);
    }
    // Zip all files.
    for($i=0;$i<count($new_files);$i++){
      #add it to the zip
      $zip->addFromString($new_files[$i],$input[$i]);
    }
    // Close zip
    $zip->close();
    // Send the file to the browser as a download
    header('Content-Disposition: attachment; filename='.basename('files_generated.zip'));
    header('Content-type: application/zip');
    readfile($tmp_file);
    unlink($tmp_file);
    exit;

  }

  /**
   * Find paths when generating files.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   * @return
   *   All path that we want generate files.
   */
  public function pathFiles(FormStateInterface $form_state) {
    // extract all path of twig files
    $extracted_twig_path_files = $this->getRealPathNamesTwigFiles();
    $module_name = $form_state->getValue('module_name');
    //Names of twig files
    $twig_files = $form_state->getValue('twig_file');
    // Filter twig files from key ( $twig_file = array($key => $value) ).
    foreach ($twig_files as $twig_file => $value) {
      $twigg_files[] = $value;
    }

    // Get all paths and name file and extension(s) for every templates.
    // name class php files
    $name_files = $this->extractNamePhpFiles($form, $form_state);
    for ($i = 0; $i < count($twigg_files); $i++) {
      // Get all path from twig files
      $path = $extracted_twig_path_files[$twigg_files[$i]];
      // Delete /templates/generator_ui from paths
      $path_file = substr($path, strpos($path, 'templates') + 10);
      // Get the right path for file creation.
      // Get all the internal path and name of the file (last element of the array).
      $path_chunks = explode('/', $path_file);
      // Delete the name of twig file from path
      array_pop($path_chunks);
      // Get the extentions & file name (last element of the array).
      $all_extensions = explode('.', $twigg_files[$i]);
      array_shift($all_extensions);
      // Remove name file.
      // If the file is module_name.install or module_name.module then delete yml extension
    if ($all_extensions[0] == 'install' || $all_extensions[0] == 'module') {
        unset($all_extensions[1]);
      }
      // Remove the .twig extention.
      array_pop($all_extensions);
      // if the path has an yml extension then generate the right file

      $pathh[$i] = implode("/", $path_chunks);
      if (substr_count($path_file, '.yml') != 0) {
        if ($pathh[$i]!=""){
          $path_ = '/'.$pathh[$i];
        }
        $final_path[] = DRUPAL_ROOT . "/modules/" . $module_name . $path_ . '/' . $module_name . '.' . implode('.', $all_extensions);
      }
      else {
        $final_path[] = DRUPAL_ROOT . "/modules/" . $module_name . '/' . $pathh[$i] . '/' . $name_files[$i] . '.' . implode('.', $all_extensions);

      }
    }
    return $final_path;
  }

  /**
   * Find paths of specific twig files.
   *
   * @return
   *   All path that we want generate files.
   */
  public function getFullPathTwig(FormStateInterface $form_state) {
    // return
    $extracted_twig_path_files = $this->getRealPathNamesTwigFiles();
    //Names of twig files
    $twig_files = $form_state->getValue('twig_file');
    // Filter twig files from key ( $twig_file = array($key => $value) ).
    foreach ($twig_files as $twig_file => $value) {
      $twigg_files[] = $value;
    }
    for ($i = 0; $i < count($twigg_files); $i++) {
      // Get all path from twig files
      $path[] = $extracted_twig_path_files[$twigg_files[$i]];
    }
    return $path;
  }

  /**
   * Find paths of all twig files.
   *
   * @return
   *   All twig files.
   */
  public function getRealPathNamesTwigFiles() {
    //path of templates files
    //find directories and files in template directory
    $dir_templates = drupal_get_path('module', 'generator_ui') . '/templates';
    $iter = new \RecursiveDirectoryIterator($dir_templates);
    foreach (new \RecursiveIteratorIterator($iter) as $file) {
      //if the file has a twig extension
      if (strpos($file->getFilename(), '.twig')) {
        $filename = $file->getFilename();
        // declare an array contains the list of twig files
        $list[$filename] = $file->getPath() . '/' . $filename;
      }
    }
    return $list;
  }

  // this method delete from an array null values
  public function is_not_null($arrays = array()) {
    foreach ($arrays as $array) {
      if ($array != NULL) {
        $new_array[] = $array;
      }
    }
    return $new_array;
  }

public static function get_module_name(FormStateInterface $form_state)
{
  return $form_state->getValue('module_name');
}
  /**
   * get All values.
   *
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   Dependencies.
   */
  public function getAllValues(FormStateInterface $form_state) {
    $values_changes = $form_state->getValues();
    foreach ($values_changes as $key => $value) {
      if (strpos($value, ",") == TRUE) {
        $values_changes[$key] = explode(",", $value);
      }
    }
    return $values_changes;
  }


  /**
   * Load and render twig files.
   *
   * @param array $params
   *   An  array containing variables which will passed in twig files.
   * @param $path
   *   The current path of twig file.
   * @return string
   *   Get the string which contains the code to generate.
   */
  protected function loadFromTwig($parameters, $pathTemplate) {
    $template = $this->twig->loadTemplate($pathTemplate);
    $output = array(
      $template->render($parameters)
    );
    $outputFinal = implode("\n", $output);

    return $outputFinal;
  }

}