<?php
/**
 * @file
 * Contains \Drupal\generator_ui\Form\GeneratorForm .
 *
 */

namespace Drupal\generator_ui\Form;


use Drupal\Core\Form\FormStateInterface;
use Drupal\generator_ui\Controller\GeneratorController;

class GeneratorConfigForm extends Generator {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'generator_config_form';

  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['into'] = array(
      '#markup' => t('<h2>' . 'Please fill the blanks to create your Config Form in D8' . '</h2>'),
      "#weight" => -3

    );
    $form['twig_file'] = array(
      "#type" => 'hidden',
      "#value" => array('form'=>'form_config.php.twig','generator.settings.yml.twig'),
    );
    $form['form'] = array(
      '#type' => 'textfield',
      '#title' => t('Name of the Config Form class'),
      '#default_value' => 'ExampleConfigForm',
      '#description' => t('Path of config Form class: modules/src/Form/'),
      '#required' => TRUE,
    );
    $form['form_id_'] = array(
      '#type' => 'textfield',
      '#title' => t('Id of the Form'),
      '#default_value' => 'id_form',
      '#required' => TRUE,
    );
    $form['validation'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Generate validation method: yes or no ?'),
    );
    $form = parent::buildForm($form, $form_state);
    $form['ajx_settings'] = array(
      '#title' => t('Config path : module/config/install/module.settings.yml'),
      '#type' => 'textarea',
      '#resizable' => TRUE,
      '#rows' => 15,
      '#attributes' => array('onclick' => 'this.select()'),
      '#cols' => 10,
      '#suffix' => '</div>'
    );
    $form['ajx']['#title'] = t('Config Form path : modules/src/Form/YourConfigForm.php');

    return $form;
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    return parent::validateForm($form, $form_state);
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    return parent::submitForm($form, $form_state);


  }


} 