<?php
/**
 * @file
 * Contains \Drupal\generator_ui\Form\GeneratorForm .
 *
 */

namespace Drupal\generator_ui\Form;


use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\generator_ui\Controller\GeneratorController;

class GeneratorForm extends Generator {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'generator_form';

  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['into'] = array(
      '#markup' => t('<h2>' . 'Please fill the blanks to create your Form in D8' . '</h2>'),
      "#weight" => -3
    );
    $form['twig_file'] = array(
      "#type" => 'hidden',
      "#value" => array('form'=>'form_simple.php.twig'),
    );
    $form['form'] = array(
      '#type' => 'textfield',
      '#title' => t('Name of the Form class'),
      '#default_value' => 'ExampleForm',
      '#description' => t('Path of Form class: module/src/Form/'),
      '#required' => TRUE,
    );
    $form['form_id_'] = array(
      '#type' => 'textfield',
      '#title' => t('Id of the Form'),
      '#default_value' => 'id_form',
      '#required' => TRUE,
    );
    $form['validation_form'] = array(
      '#type' => 'checkbox',
      '#title' => $this->t('Generate validation method: yes or no ?'),
    );
    $form['rebuild_form'] = array(
      '#type' => 'checkbox',
      "#title" => $this->t("Check to sets the form to be rebuilt after processing"),
    );
    $form['validate_number'] = array(
      '#type' => 'checkbox',
      "#title" => $this->t("Check to generate element_validate"),
    );
    $form['validate_states'] = array(
      '#type' => 'checkbox',
      "#title" => $this->t("Check to generate states"),
    );


    $form = parent::buildForm($form, $form_state);
    $form['ajx']['#suffix'] = '</div>';
    return $form;

  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
    return parent::validateForm($form, $form_state);
  }
  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    return parent::submitForm($form, $form_state);


  }


}