<?php

/**
 * @file
 * Contains \Drupal\generator_ui\Form\GeneratorRouting.
 *
 */

namespace Drupal\generator_ui\Form;


//Use the necessary libraries
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\OpenModalDialogCommand;
use Drupal\Core\Form\FormStateInterface;
use Drupal\generator_ui\Controller\GeneratorController;


class GeneratorRouting extends Generator {

  /**
   * Returns a unique string identifying the form.
   *
   * @return string
   *   The unique string identifying the form.
   */
  public function getFormId() {
    return 'routing';
  }

  /**
   * Form constructor.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   The form structure.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form['into'] = array(
      '#markup' => t('<h2>' . 'Please fill the blanks to create your routing, Controllers and form files' . '</h2>'),
      "#weight" => -2
    );
    $form['type_routing'] = array(
      '#type' => 'select',
      '#title' => t('Routing\'s type'),
      '#options' => array('controller' => 'Controller', 'form_' => 'Form'),
      '#executes_submit_callback' => FALSE,
      '#ajax' => array(
        'callback' => '::choiceRouting',
        'wrapper' => 'ajax_choice',
      ),
    );
    $form['route'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Route name'),
      '#default_value' => 'module.key',
      '#required' => TRUE,
      '#description' => t('The route name <strong>must</strong> be unique,so the route name must start with the module name followed by an unique key'),
    );
    $form['pathh'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Path'),
      '#default_value' => '/hello',
      '#required' => TRUE,
      '#description' => t('URL of the page,Ex :www.example.com/module/example/<strong>arguments</strong>'),
    );
    $form['argums'] = array(
      '#type' => 'textfield',
      '#title' => $this->t('Arguments in path'),
      '#description' => t('Arguments must be separated by / and putted in braces,Ex: /{first_argument}/{second_argument}'),
    );
    $form['title'] = array(
      '#type' => 'textfield',
      '#default_value' => 'Page title',
      '#title' => $this->t('Page title'),
    );
    $form['control_method'] = array(
      '#type' => 'fieldset',
      '#title' => t('The controller & its method'),
      '#collapsible' => TRUE,
      '#prefix' => '<div id="ajax_choice">',
      '#suffix' => '</div>',
    );
    if ($form_state->getValue('type_routing') == "form_") {
      $form['control_form'] = array(
        '#type' => 'fieldset',
        '#title' => t('The form'),
        '#collapsible' => TRUE,
        '#prefix' => '<div id="ajax_choice">',
        '#suffix' => '</div>',
      );
      $form['twig_file'] = array(
        "#type" => 'hidden',
        "#value" => array(
          'form' => 'form.php.twig',
          'generator.routing.yml.twig',
        ),
      );
      $form['control_form']['form'] = array(
        '#type' => 'textfield',
        '#default_value' => 'ExampleForm',
        '#title' => $this->t('Name of the form class'),
        '#description' => t('Path of form class: module/src/Form'),
      );
      $form['control_form']['form_id_'] = array(
        '#type' => 'textfield',
        '#default_value' => 'form_id',
        '#title' => $this->t('Form id'),
      );
    }
    else {
      $form['twig_file'] = array(
        "#type" => 'hidden',
        "#value" => array(
          'controller' => 'controller.php.twig',
          'generator.routing.yml.twig',
        ),
      );
      $form['control_method']['controller'] = array(
        '#type' => 'textfield',
        '#title' => $this->t('Name of the controller class'),
        '#default_value' => 'ExampleController',
        '#description' => t('Path of controller class: module/src/Controller'),
      );
      $form['control_method']['method'] = array(
        '#type' => 'textfield',
        '#default_value' => 'hello',
        '#title' => $this->t('method that exists in the controller'),
      );
   }
    $generate_permission = t('<strong> <a href="!permissions">Click here to generate .permissions.yml file</a></strong>', array('!permissions' => \Drupal::url('generator.permissions')));
    $form['permission'] = array(
      '#type' => 'textfield',
      '#title' => 'Permission',
      '#default_value' => 'access content',
      '#description' => $this->t('If you declare a permission in module.permissions.yml file ,you can use it (' . $generate_permission . ' ), else you can declare a permission existent in the core.Ex: access content'),
    );
    $form = parent::buildForm($form, $form_state);
    $form['ajx_routing'] = array(
      '#title' => t('Routing path : module/module.routing.yml'),
      '#type' => 'textarea',
      '#resizable' => TRUE,
      '#rows' => 15,
      '#attributes' => array('onclick' => 'this.select()'),
      '#cols' => 10,
      '#suffix' => '</div>',
    );

    $form['ajx']['#title'] = t('Controller path : module/src/Controller/YourController.php OR Form path : module/src/Form/YourForm.php');
    return $form;
  }

  public function redirectModal() {
    $response = new AjaxResponse();
    $form_permission = \Drupal::formBuilder()
      ->getForm('\Drupal\generator_ui\Form\GenerePermissionsForm');
    $response->addCommand(new OpenModalDialogCommand('Add a permission', \Drupal::service('renderer')
      ->render($form_permission)));

    return $response;
  }

  /**
   * return type of routing: Form routing and controller routing.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   *
   * @return array
   *   return choice of routing form.
   */
  public function choiceRouting(array $form, FormStateInterface $form_state) {
    $choiceRouting = $form_state->getValue('type_routing');
    switch ($choiceRouting) {
      case "controller" :
        return $form['control_method'];
        break;
      case "form_" :
        return $form['control_form'];
        break;
    }
  }
  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

    return parent::validateForm($form, $form_state);
  }

  /**
   * Form submission handler.
   *
   * @param array $form
   *   An associative array containing the structure of the form.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The current state of the form.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

    return parent::submitForm($form, $form_state);

  }

}